@@ -104,7 +104,7 @@ static const struct rd_kafka_feature_map {
 		/* @brief >=0.8.2.0: Broker-based Group coordinator */
 		.feature = RD_KAFKA_FEATURE_BROKER_GROUP_COORD,
 		.depends = {
-			{ RD_KAFKAP_GroupCoordinator, 0, 0 },
+			{ RD_KAFKAP_FindCoordinator, 0, 0 },
 			{ -1 },
 		},
 	},
@@ -112,7 +112,7 @@ static const struct rd_kafka_feature_map {
 		/* @brief >=0.9.0: Broker-based balanced consumer groups. */
 		.feature = RD_KAFKA_FEATURE_BROKER_BALANCED_CONSUMER,
 		.depends = {
-			{ RD_KAFKAP_GroupCoordinator, 0, 0 },
+			{ RD_KAFKAP_FindCoordinator, 0, 0 },
 			{ RD_KAFKAP_OffsetCommit, 1, 2 },
 			{ RD_KAFKAP_OffsetFetch, 1, 1 },
 			{ RD_KAFKAP_JoinGroup, 0, 0 },
@@ -163,7 +163,7 @@ static const struct rd_kafka_feature_map {
                  * GrooupCoordinator was released in 0.8.2 */
                 .feature = RD_KAFKA_FEATURE_LZ4,
                 .depends = {
-                        { RD_KAFKAP_GroupCoordinator, 0, 0 },
+                        { RD_KAFKAP_FindCoordinator, 0, 0 },
                         { -1 },
                 },
         },
@@ -226,7 +226,7 @@ static struct rd_kafka_ApiVersion rd_kafka_ApiVersion_0_9_0[] = {
 	{ RD_KAFKAP_Metadata, 0, 0 },
 	{ RD_KAFKAP_OffsetCommit, 0, 2 },
 	{ RD_KAFKAP_OffsetFetch, 0, 1 },
-	{ RD_KAFKAP_GroupCoordinator, 0, 0 },
+	{ RD_KAFKAP_FindCoordinator, 0, 0 },
 	{ RD_KAFKAP_JoinGroup, 0, 0 },
 	{ RD_KAFKAP_Heartbeat, 0, 0 },
 	{ RD_KAFKAP_LeaveGroup, 0, 0 },
@@ -243,7 +243,7 @@ static struct rd_kafka_ApiVersion rd_kafka_ApiVersion_0_8_2[] = {
 	{ RD_KAFKAP_Metadata, 0, 0 },
 	{ RD_KAFKAP_OffsetCommit, 0, 1 },
 	{ RD_KAFKAP_OffsetFetch, 0, 1 },
-	{ RD_KAFKAP_GroupCoordinator, 0, 0 }
+	{ RD_KAFKAP_FindCoordinator, 0, 0 }
 };
 
 /* =~ 0.8.1 */
