@@ -35,6 +35,10 @@
 #include "rdkafka_header.h"
 #include "rdkafka_lz4.h"
 
+#if WITH_ZSTD
+#include "rdkafka_zstd.h"
+#endif
+
 #include "snappy.h"
 #include "rdvarint.h"
 #include "crc32c.h"
@@ -46,6 +50,7 @@ typedef struct rd_kafka_msgset_writer_s {
         int16_t msetw_ApiVersion;        /* ProduceRequest ApiVersion */
         int     msetw_MsgVersion;        /* MsgVersion to construct */
         int     msetw_features;          /* Protocol features to use */
+        rd_kafka_compression_t msetw_compression; /**< Compression type */
         int     msetw_msgcntmax;         /* Max number of messages to send
                                           * in a batch. */
         size_t  msetw_messages_len;      /* Total size of Messages, with Message
@@ -65,16 +70,22 @@ typedef struct rd_kafka_msgset_writer_s {
         int64_t msetw_MaxTimestamp;      /* Maximum timestamp in batch */
         size_t  msetw_of_CRC;            /* offset of MessageSet.CRC */
 
+        rd_kafka_msgbatch_t *msetw_batch; /**< Convenience pointer to
+                                           *   rkbuf_u.Produce.batch */
+
         /* First message information */
         struct {
                 size_t     of;  /* rkbuf's first message position */
                 int64_t    timestamp;
         } msetw_firstmsg;
 
+        rd_kafka_pid_t msetw_pid;        /**< Idempotent producer's
+                                          *   current Producer Id */
         rd_kafka_broker_t *msetw_rkb;    /* @warning Not a refcounted
                                           *          reference! */
         rd_kafka_toppar_t *msetw_rktp;   /* @warning Not a refcounted
                                           *          reference! */
+        rd_kafka_msgq_t *msetw_msgq;     /**< Input message queue */
 } rd_kafka_msgset_writer_t;
 
 
@@ -88,25 +99,106 @@ typedef struct rd_kafka_msgset_writer_s {
 static RD_INLINE void
 rd_kafka_msgset_writer_select_MsgVersion (rd_kafka_msgset_writer_t *msetw) {
         rd_kafka_broker_t *rkb = msetw->msetw_rkb;
+        rd_kafka_toppar_t *rktp = msetw->msetw_rktp;
+        int16_t min_ApiVersion = 0;
         int feature;
+        /* Map compression types to required feature and ApiVersion */
+        static const struct {
+                int feature;
+                int16_t ApiVersion;
+        } compr_req[RD_KAFKA_COMPRESSION_NUM] = {
+                [RD_KAFKA_COMPRESSION_LZ4] = { RD_KAFKA_FEATURE_LZ4, 0 },
+#if WITH_ZSTD
+                [RD_KAFKA_COMPRESSION_ZSTD] = { RD_KAFKA_FEATURE_ZSTD, 7 },
+#endif
+        };
 
         if ((feature = rkb->rkb_features & RD_KAFKA_FEATURE_MSGVER2)) {
-                msetw->msetw_ApiVersion = 3;
+                min_ApiVersion = 3;
                 msetw->msetw_MsgVersion = 2;
                 msetw->msetw_features |= feature;
         } else if ((feature = rkb->rkb_features & RD_KAFKA_FEATURE_MSGVER1)) {
-                msetw->msetw_ApiVersion = 2;
+                min_ApiVersion = 2;
                 msetw->msetw_MsgVersion = 1;
                 msetw->msetw_features |= feature;
         } else {
                 if ((feature =
                      rkb->rkb_features & RD_KAFKA_FEATURE_THROTTLETIME)) {
-                        msetw->msetw_ApiVersion = 1;
+                        min_ApiVersion = 1;
                         msetw->msetw_features |= feature;
                 } else
-                        msetw->msetw_ApiVersion = 0;
+                        min_ApiVersion = 0;
                 msetw->msetw_MsgVersion = 0;
         }
+
+        msetw->msetw_compression = rktp->rktp_rkt->rkt_conf.compression_codec;
+
+        /*
+         * Check that the configured compression type is supported
+         * by both client and broker, else disable compression.
+         */
+        if (msetw->msetw_compression &&
+            (rd_kafka_broker_ApiVersion_supported(
+                    rkb, RD_KAFKAP_Produce,
+                    0, compr_req[msetw->msetw_compression].ApiVersion,
+                    NULL) == -1 ||
+             (compr_req[msetw->msetw_compression].feature &&
+              !(msetw->msetw_rkb->rkb_features &
+                compr_req[msetw->msetw_compression].feature)))) {
+                if (unlikely(rd_interval(
+                                     &rkb->rkb_suppress.unsupported_compression,
+                                     /* at most once per day */
+                                     (rd_ts_t)86400 * 1000 * 1000, 0) > 0))
+                        rd_rkb_log(rkb, LOG_NOTICE, "COMPRESSION",
+                                   "%.*s [%"PRId32"]: "
+                                   "Broker does not support compression "
+                                   "type %s: not compressing batch",
+                                   RD_KAFKAP_STR_PR(rktp->rktp_rkt->rkt_topic),
+                                   rktp->rktp_partition,
+                                   rd_kafka_compression2str(
+                                           msetw->msetw_compression));
+                else
+                        rd_rkb_dbg(rkb, MSG, "PRODUCE",
+                                   "%.*s [%"PRId32"]: "
+                                   "Broker does not support compression "
+                                   "type %s: not compressing batch",
+                                   RD_KAFKAP_STR_PR(rktp->rktp_rkt->rkt_topic),
+                                   rktp->rktp_partition,
+                                   rd_kafka_compression2str(
+                                           msetw->msetw_compression));
+
+                msetw->msetw_compression = RD_KAFKA_COMPRESSION_NONE;
+        } else {
+                /* Broker supports this compression type. */
+                msetw->msetw_features |=
+                        compr_req[msetw->msetw_compression].feature;
+
+                if (min_ApiVersion <
+                    compr_req[msetw->msetw_compression].ApiVersion)
+                        min_ApiVersion =
+                                compr_req[msetw->msetw_compression].ApiVersion;
+        }
+
+        /* MsgVersion specific setup. */
+        switch (msetw->msetw_MsgVersion)
+        {
+        case 2:
+                msetw->msetw_relative_offsets = 1; /* OffsetDelta */
+                break;
+        case 1:
+                if (msetw->msetw_compression != RD_KAFKA_COMPRESSION_NONE)
+                        msetw->msetw_relative_offsets = 1;
+                break;
+        }
+
+        /* Set the highest ApiVersion supported by us and broker */
+        msetw->msetw_ApiVersion = rd_kafka_broker_ApiVersion_supported(
+                rkb,
+                RD_KAFKAP_Produce, min_ApiVersion, 7, NULL);
+
+        /* It should not be possible to get a lower version than requested,
+         * otherwise the logic in this function is buggy. */
+        rd_assert(msetw->msetw_ApiVersion >= min_ApiVersion);
 }
 
 
@@ -149,9 +241,13 @@ rd_kafka_msgset_writer_alloc_buf (rd_kafka_msgset_writer_t *msetw) {
          */
         switch (msetw->msetw_ApiVersion)
         {
+        case 7:
+        case 6:
+        case 5:
+        case 4:
         case 3:
                 /* Add TransactionalId */
-                hdrsize += RD_KAFKAP_STR_SIZE(rk->rk_eos.TransactionalId);
+                hdrsize += RD_KAFKAP_STR_SIZE(rk->rk_eos.transactional_id);
                 /* FALLTHRU */
         case 0:
         case 1:
@@ -221,8 +317,7 @@ rd_kafka_msgset_writer_alloc_buf (rd_kafka_msgset_writer_t *msetw) {
          * space for each message to be copied based on this limit.
          */
         if (rk->rk_conf.msg_copy_max_size > 0) {
-                size_t queued_bytes = rd_kafka_msgq_size(&msetw->msetw_rktp->
-                                                         rktp_xmit_msgq);
+                size_t queued_bytes = rd_kafka_msgq_size(msetw->msetw_msgq);
                 bufsize += RD_MIN(queued_bytes,
                                   (size_t)rk->rk_conf.msg_copy_max_size *
                                   msetw->msetw_msgcntmax);
@@ -258,7 +353,6 @@ static void
 rd_kafka_msgset_writer_write_MessageSet_v2_header (
         rd_kafka_msgset_writer_t *msetw) {
         rd_kafka_buf_t *rkbuf = msetw->msetw_rkbuf;
-        rd_kafka_t *rk = msetw->msetw_rkb->rkb_rk;
 
         rd_kafka_assert(NULL, msetw->msetw_ApiVersion >= 3);
         rd_kafka_assert(NULL, msetw->msetw_MsgVersion == 2);
@@ -295,12 +389,12 @@ rd_kafka_msgset_writer_write_MessageSet_v2_header (
         rd_kafka_buf_write_i64(rkbuf, 0);
 
         /* ProducerId */
-        rd_kafka_buf_write_i64(rkbuf, rk->rk_eos.PID);
+        rd_kafka_buf_write_i64(rkbuf, msetw->msetw_pid.id);
 
         /* ProducerEpoch */
-        rd_kafka_buf_write_i16(rkbuf, rk->rk_eos.ProducerEpoch);
+        rd_kafka_buf_write_i16(rkbuf, msetw->msetw_pid.epoch);
 
-        /* BaseSequence */
+        /* BaseSequence: updated later in case of Idempotent Producer */
         rd_kafka_buf_write_i32(rkbuf, -1);
 
         /* RecordCount: udpated later */
@@ -323,8 +417,8 @@ rd_kafka_msgset_writer_write_Produce_header (rd_kafka_msgset_writer_t *msetw) {
         rd_kafka_itopic_t *rkt = msetw->msetw_rktp->rktp_rkt;
 
         /* V3: TransactionalId */
-        if (msetw->msetw_ApiVersion == 3)
-                rd_kafka_buf_write_kstr(rkbuf, rk->rk_eos.TransactionalId);
+        if (msetw->msetw_ApiVersion >= 3)
+                rd_kafka_buf_write_kstr(rkbuf, rk->rk_eos.transactional_id);
 
         /* RequiredAcks */
         rd_kafka_buf_write_i16(rkbuf, rkt->rkt_conf.required_acks);
@@ -372,9 +466,11 @@ rd_kafka_msgset_writer_write_Produce_header (rd_kafka_msgset_writer_t *msetw) {
  * @locality broker thread
  */
 static int rd_kafka_msgset_writer_init (rd_kafka_msgset_writer_t *msetw,
-                                         rd_kafka_broker_t *rkb,
-                                         rd_kafka_toppar_t *rktp) {
-        int msgcnt = rktp->rktp_xmit_msgq.rkmq_msg_cnt;
+                                        rd_kafka_broker_t *rkb,
+                                        rd_kafka_toppar_t *rktp,
+                                        rd_kafka_msgq_t *rkmq,
+                                        rd_kafka_pid_t pid) {
+        int msgcnt = rd_kafka_msgq_len(rkmq);
 
         if (msgcnt == 0)
                 return 0;
@@ -383,6 +479,8 @@ static int rd_kafka_msgset_writer_init (rd_kafka_msgset_writer_t *msetw,
 
         msetw->msetw_rktp = rktp;
         msetw->msetw_rkb = rkb;
+        msetw->msetw_msgq = rkmq;
+        msetw->msetw_pid = pid;
 
         /* Max number of messages to send in a batch,
          * limited by current queue size or configured batch size,
@@ -395,18 +493,6 @@ static int rd_kafka_msgset_writer_init (rd_kafka_msgset_writer_t *msetw,
         /* Select MsgVersion to use */
         rd_kafka_msgset_writer_select_MsgVersion(msetw);
 
-        /* MsgVersion specific setup. */
-        switch (msetw->msetw_MsgVersion)
-        {
-        case 2:
-                msetw->msetw_relative_offsets = 1; /* OffsetDelta */
-                break;
-        case 1:
-                if (rktp->rktp_rkt->rkt_conf.compression_codec)
-                        msetw->msetw_relative_offsets = 1;
-                break;
-        }
-
         /* Allocate backing buffer */
         rd_kafka_msgset_writer_alloc_buf(msetw);
 
@@ -420,6 +506,10 @@ static int rd_kafka_msgset_writer_init (rd_kafka_msgset_writer_t *msetw,
         msetw->msetw_firstmsg.of = rd_buf_write_pos(&msetw->msetw_rkbuf->
                                                     rkbuf_buf);
 
+        rd_kafka_msgbatch_init(&msetw->msetw_rkbuf->rkbuf_u.Produce.batch,
+                               rktp, pid);
+        msetw->msetw_batch = &msetw->msetw_rkbuf->rkbuf_u.Produce.batch;
+
         return msetw->msetw_msgcntmax;
 }
 
@@ -708,11 +798,12 @@ rd_kafka_msgset_writer_write_msg (rd_kafka_msgset_writer_t *msetw,
  *        the messageset.
  *
  *        May not write any messages.
+ *
+ * @returns 1 on success or 0 on error.
  */
-static void
+static int
 rd_kafka_msgset_writer_write_msgq (rd_kafka_msgset_writer_t *msetw,
                                    rd_kafka_msgq_t *rkmq) {
-        rd_kafka_buf_t *rkbuf = msetw->msetw_rkbuf;
         rd_kafka_toppar_t *rktp = msetw->msetw_rktp;
         rd_kafka_broker_t *rkb = msetw->msetw_rkb;
         size_t len = rd_buf_len(&msetw->msetw_rkbuf->rkbuf_buf);
@@ -727,25 +818,55 @@ rd_kafka_msgset_writer_write_msgq (rd_kafka_msgset_writer_t *msetw,
         /* Internal latency calculation base.
          * Uses rkm_ts_timeout which is enqueue time + timeout */
         int_latency_base = now +
-                (rktp->rktp_rkt->rkt_conf.message_timeout_ms * 1000);
+                ((rd_ts_t) rktp->rktp_rkt->rkt_conf.message_timeout_ms * 1000);
 
         /* Acquire BaseTimestamp from first message. */
         rkm = TAILQ_FIRST(&rkmq->rkmq_msgs);
         rd_kafka_assert(NULL, rkm);
         msetw->msetw_firstmsg.timestamp = rkm->rkm_timestamp;
 
+        rd_kafka_msgbatch_set_first_msg(msetw->msetw_batch, rkm);
+
         /*
          * Write as many messages as possible until buffer is full
          * or limit reached.
          */
         do {
-                if (unlikely(msgcnt == msetw->msetw_msgcntmax ||
-                             len + rd_kafka_msg_wire_size(rkm, msetw->
-                                                          msetw_MsgVersion) >
-                             max_msg_size)) {
+                if (unlikely(msetw->msetw_batch->last_msgid &&
+                             msetw->msetw_batch->last_msgid <
+                             rkm->rkm_u.producer.msgid)) {
                         rd_rkb_dbg(rkb, MSG, "PRODUCE",
+                                   "%.*s [%"PRId32"]: "
+                                   "Reconstructed MessageSet "
+                                   "(%d message(s), %"PRIusz" bytes, "
+                                   "MsgIds %"PRIu64"..%"PRIu64")",
+                                   RD_KAFKAP_STR_PR(rktp->rktp_rkt->rkt_topic),
+                                   rktp->rktp_partition,
+                                   msgcnt, len,
+                                   msetw->msetw_batch->first_msgid,
+                                   msetw->msetw_batch->last_msgid);
+                        break;
+                }
+
+                /* Check if there is enough space in the current messageset
+                 * to add this message.
+                 * Since calculating the total size of a request at produce()
+                 * time is tricky (we don't know the protocol version or
+                 * MsgVersion that will be used), we allow a messageset to
+                 * overshoot the message.max.bytes limit by one message to
+                 * avoid getting stuck here.
+                 * The actual messageset size is enforced by the broker. */
+                if (unlikely(msgcnt == msetw->msetw_msgcntmax ||
+                             (msgcnt > 0 &&
+                              len + rd_kafka_msg_wire_size(rkm, msetw->
+                                                           msetw_MsgVersion) >
+                              max_msg_size))) {
+                        rd_rkb_dbg(rkb, MSG, "PRODUCE",
+                                   "%.*s [%"PRId32"]: "
                                    "No more space in current MessageSet "
                                    "(%i message(s), %"PRIusz" bytes)",
+                                   RD_KAFKAP_STR_PR(rktp->rktp_rkt->rkt_topic),
+                                   rktp->rktp_partition,
                                    msgcnt, len);
                         break;
                 }
@@ -758,7 +879,7 @@ rd_kafka_msgset_writer_write_msgq (rd_kafka_msgset_writer_t *msetw,
 
                 /* Move message to buffer's queue */
                 rd_kafka_msgq_deq(rkmq, rkm, 1);
-                rd_kafka_msgq_enq(&rkbuf->rkbuf_msgq, rkm);
+                rd_kafka_msgq_enq(&msetw->msetw_batch->msgq, rkm);
 
                 msetw->msetw_messages_kvlen += rkm->rkm_len + rkm->rkm_key_len;
 
@@ -774,12 +895,43 @@ rd_kafka_msgset_writer_write_msgq (rd_kafka_msgset_writer_t *msetw,
                 len += rd_kafka_msgset_writer_write_msg(msetw, rkm, msgcnt, 0,
                                                         NULL);
 
-                rd_dassert(len <= max_msg_size);
                 msgcnt++;
 
         } while ((rkm = TAILQ_FIRST(&rkmq->rkmq_msgs)));
 
         msetw->msetw_MaxTimestamp = MaxTimestamp;
+
+        /* Idempotent Producer:
+         * When reconstructing a batch to retry make sure
+         * the original message sequence span matches identically
+         * or we can't guarantee exactly-once delivery.
+         * If this check fails we raise a fatal error since
+         * it is unrecoverable and most likely caused by a bug
+         * in the client implementation. */
+        if (msgcnt > 0 && msetw->msetw_batch->last_msgid) {
+                rd_kafka_msg_t *lastmsg;
+
+                lastmsg = rd_kafka_msgq_last(&msetw->msetw_batch->msgq);
+                rd_assert(lastmsg);
+
+                if (unlikely(lastmsg->rkm_u.producer.msgid !=
+                             msetw->msetw_batch->last_msgid)) {
+                        rd_kafka_set_fatal_error(
+                                rkb->rkb_rk,
+                                RD_KAFKA_RESP_ERR__INCONSISTENT,
+                                "Unable to reconstruct MessageSet "
+                                "(currently with %d message(s)) "
+                                "with msgid range %"PRIu64"..%"PRIu64": "
+                                "last message added has msgid %"PRIu64": "
+                                "unable to guarantee consistency",
+                                msgcnt,
+                                msetw->msetw_batch->first_msgid,
+                                msetw->msetw_batch->last_msgid,
+                                lastmsg->rkm_u.producer.msgid);
+                        return 0;
+                }
+        }
+        return 1;
 }
 
 
@@ -801,7 +953,7 @@ rd_kafka_msgset_writer_compress_gzip (rd_kafka_msgset_writer_t *msetw,
         int r;
         int comp_level =
                 msetw->msetw_rktp->rktp_rkt->rkt_conf.compression_level;
-				
+
         memset(&strm, 0, sizeof(strm));
         r = deflateInit2(&strm, comp_level,
                          Z_DEFLATED, 15+16,
@@ -948,7 +1100,22 @@ rd_kafka_msgset_writer_compress_lz4 (rd_kafka_msgset_writer_t *msetw,
         return (err ? -1 : 0);
 }
 
-
+#if WITH_ZSTD
+/**
+ * @brief Compress messageset using ZSTD
+ */
+static int
+rd_kafka_msgset_writer_compress_zstd (rd_kafka_msgset_writer_t *msetw,
+                                     rd_slice_t *slice, struct iovec *ciov) {
+        rd_kafka_resp_err_t err;
+        int comp_level =
+                msetw->msetw_rktp->rktp_rkt->rkt_conf.compression_level;
+        err = rd_kafka_zstd_compress(msetw->msetw_rkb,
+                                    comp_level,
+                                    slice, &ciov->iov_base, &ciov->iov_len);
+        return (err ? -1 : 0);
+}
+#endif
 
 /**
  * @brief Compress the message set.
@@ -961,7 +1128,6 @@ rd_kafka_msgset_writer_compress_lz4 (rd_kafka_msgset_writer_t *msetw,
 static int
 rd_kafka_msgset_writer_compress (rd_kafka_msgset_writer_t *msetw,
                                  size_t *outlenp) {
-        rd_kafka_toppar_t *rktp = msetw->msetw_rktp;
         rd_buf_t *rbuf = &msetw->msetw_rkbuf->rkbuf_buf;
         rd_slice_t slice;
         size_t len = *outlenp;
@@ -975,7 +1141,7 @@ rd_kafka_msgset_writer_compress (rd_kafka_msgset_writer_t *msetw,
         r = rd_slice_init(&slice, rbuf, msetw->msetw_firstmsg.of, len);
         rd_assert(r == 0 || !*"invalid firstmsg position");
 
-        switch (rktp->rktp_rkt->rkt_conf.compression_codec)
+        switch (msetw->msetw_compression)
         {
 #if WITH_ZLIB
         case RD_KAFKA_COMPRESSION_GZIP:
@@ -991,13 +1157,14 @@ rd_kafka_msgset_writer_compress (rd_kafka_msgset_writer_t *msetw,
 #endif
 
         case RD_KAFKA_COMPRESSION_LZ4:
-                /* Skip LZ4 compression if broker doesn't support it. */
-                if (!(msetw->msetw_rkb->rkb_features & RD_KAFKA_FEATURE_LZ4))
-                        return -1;
-
                 r = rd_kafka_msgset_writer_compress_lz4(msetw, &slice, &ciov);
                 break;
 
+#if WITH_ZSTD
+        case RD_KAFKA_COMPRESSION_ZSTD:
+                r = rd_kafka_msgset_writer_compress_zstd(msetw, &slice, &ciov);
+                break;
+#endif
 
         default:
                 rd_kafka_assert(NULL,
@@ -1017,7 +1184,7 @@ rd_kafka_msgset_writer_compress (rd_kafka_msgset_writer_t *msetw,
         }
 
         /* Set compression codec in MessageSet.Attributes */
-        msetw->msetw_Attributes |= rktp->rktp_rkt->rkt_conf.compression_codec;
+        msetw->msetw_Attributes |= msetw->msetw_compression;
 
         /* Rewind rkbuf to the pre-message checkpoint (firstmsg)
          * and replace the original message(s) with the compressed payload,
@@ -1044,7 +1211,7 @@ rd_kafka_msgset_writer_compress (rd_kafka_msgset_writer_t *msetw,
                 };
                 outlen = rd_kafka_msgset_writer_write_msg(
                         msetw, &rkm, 0,
-                        rktp->rktp_rkt->rkt_conf.compression_codec,
+                        msetw->msetw_compression,
                         rd_free/*free for ciov.iov_base*/);
         }
 
@@ -1085,7 +1252,7 @@ static void
 rd_kafka_msgset_writer_finalize_MessageSet_v2_header (
         rd_kafka_msgset_writer_t *msetw) {
         rd_kafka_buf_t *rkbuf = msetw->msetw_rkbuf;
-        int msgcnt = rd_kafka_msgq_len(&rkbuf->rkbuf_msgq);
+        int msgcnt = rd_kafka_msgq_len(&rkbuf->rkbuf_batch.msgq);
 
         rd_kafka_assert(NULL, msgcnt > 0);
         rd_kafka_assert(NULL, msetw->msetw_ApiVersion >= 3);
@@ -1117,6 +1284,10 @@ rd_kafka_msgset_writer_finalize_MessageSet_v2_header (
                                 RD_KAFKAP_MSGSET_V2_OF_MaxTimestamp,
                                 msetw->msetw_MaxTimestamp);
 
+        rd_kafka_buf_update_i32(rkbuf, msetw->msetw_of_start +
+                                RD_KAFKAP_MSGSET_V2_OF_BaseSequence,
+                                msetw->msetw_batch->first_seq);
+
         rd_kafka_buf_update_i32(rkbuf, msetw->msetw_of_start +
                                 RD_KAFKAP_MSGSET_V2_OF_RecordCount, msgcnt);
 
@@ -1170,7 +1341,8 @@ rd_kafka_msgset_writer_finalize (rd_kafka_msgset_writer_t *msetw,
         int cnt;
 
         /* No messages added, bail out early. */
-        if (unlikely((cnt = rd_kafka_msgq_len(&rkbuf->rkbuf_msgq)) == 0)) {
+        if (unlikely((cnt =
+                      rd_kafka_msgq_len(&rkbuf->rkbuf_batch.msgq)) == 0)) {
                 rd_kafka_buf_destroy(rkbuf);
                 return NULL;
         }
@@ -1184,9 +1356,17 @@ rd_kafka_msgset_writer_finalize (rd_kafka_msgset_writer_t *msetw,
         rd_atomic64_add(&rktp->rktp_c.tx_msgs, cnt);
         rd_atomic64_add(&rktp->rktp_c.tx_msg_bytes, msetw->msetw_messages_kvlen);
 
+        /* Idempotent Producer:
+         * Store request's PID for matching on response
+         * if the instance PID has changed and thus made
+         * the request obsolete. */
+        msetw->msetw_rkbuf->rkbuf_u.Produce.batch.pid = msetw->msetw_pid;
+
         /* Compress the message set */
-        if (rktp->rktp_rkt->rkt_conf.compression_codec)
-                rd_kafka_msgset_writer_compress(msetw, &len);
+        if (msetw->msetw_compression) {
+                if (rd_kafka_msgset_writer_compress(msetw, &len) == -1)
+                        msetw->msetw_compression = 0;
+        }
 
         msetw->msetw_messages_len = len;
 
@@ -1199,10 +1379,22 @@ rd_kafka_msgset_writer_finalize (rd_kafka_msgset_writer_t *msetw,
         rd_rkb_dbg(msetw->msetw_rkb, MSG, "PRODUCE",
                    "%s [%"PRId32"]: "
                    "Produce MessageSet with %i message(s) (%"PRIusz" bytes, "
-                   "ApiVersion %d, MsgVersion %d)",
+                   "ApiVersion %d, MsgVersion %d, MsgId %"PRIu64", "
+                   "BaseSeq %"PRId32", %s, %s)",
                    rktp->rktp_rkt->rkt_topic->str, rktp->rktp_partition,
                    cnt, msetw->msetw_MessageSetSize,
-                   msetw->msetw_ApiVersion, msetw->msetw_MsgVersion);
+                   msetw->msetw_ApiVersion, msetw->msetw_MsgVersion,
+                   msetw->msetw_batch->first_msgid,
+                   msetw->msetw_batch->first_seq,
+                   rd_kafka_pid2str(msetw->msetw_pid),
+                   msetw->msetw_compression ?
+                   rd_kafka_compression2str(msetw->msetw_compression) :
+                   "uncompressed");
+
+        rd_kafka_msgq_verify_order(rktp, &msetw->msetw_batch->msgq,
+                                   msetw->msetw_batch->first_msgid, rd_false);
+
+        rd_kafka_msgbatch_ready_produce(msetw->msetw_batch);
 
         return rkbuf;
 }
@@ -1225,14 +1417,22 @@ rd_kafka_msgset_writer_finalize (rd_kafka_msgset_writer_t *msetw,
 rd_kafka_buf_t *
 rd_kafka_msgset_create_ProduceRequest (rd_kafka_broker_t *rkb,
                                        rd_kafka_toppar_t *rktp,
+                                       rd_kafka_msgq_t *rkmq,
+                                       const rd_kafka_pid_t pid,
                                        size_t *MessageSetSizep) {
 
         rd_kafka_msgset_writer_t msetw;
 
-        if (rd_kafka_msgset_writer_init(&msetw, rkb, rktp) == 0)
+        if (rd_kafka_msgset_writer_init(&msetw, rkb, rktp, rkmq, pid) == 0)
                 return NULL;
 
-        rd_kafka_msgset_writer_write_msgq(&msetw, &rktp->rktp_xmit_msgq);
+        if (!rd_kafka_msgset_writer_write_msgq(&msetw, msetw.msetw_msgq)) {
+                /* Error while writing messages to MessageSet,
+                 * move all messages back on the xmit queue. */
+                rd_kafka_msgq_insert_msgq(
+                        rkmq, &msetw.msetw_batch->msgq,
+                        rktp->rktp_rkt->rkt_conf.msg_order_cmp);
+        }
 
         return rd_kafka_msgset_writer_finalize(&msetw, MessageSetSizep);
 }
