@@ -147,7 +147,26 @@ void rd_usleep (int usec, rd_atomic32_t *terminate) {
 #define rd_gettimeofday(tv,tz)  gettimeofday(tv,tz)
 
 
+#ifndef __COVERITY__
 #define rd_assert(EXPR)  assert(EXPR)
+#else
+extern void __coverity_panic__(void);
+#define rd_assert(EXPR) do {                    \
+                if (!(EXPR))                    \
+                        __coverity_panic__();   \
+        }
+#endif
+
+
+static RD_INLINE RD_UNUSED
+const char *rd_getenv (const char *env, const char *def) {
+        const char *tmp;
+        tmp = getenv(env);
+        if (tmp && *tmp)
+                return tmp;
+        return def;
+}
+
 
 /**
  * Empty struct initializer
@@ -158,6 +177,25 @@ void rd_usleep (int usec, rd_atomic32_t *terminate) {
  * Sockets, IO
  */
 
+ /** @brief Socket type */
+typedef int rd_socket_t;
+
+/** @brief Socket API error return value */
+#define RD_SOCKET_ERROR (-1)
+
+/** @brief Last socket error */
+#define rd_socket_errno errno
+
+
+/** @brief String representation of socket error */
+#define rd_socket_strerror(ERR) rd_strerror(ERR)
+
+/** @brief poll() struct type */
+typedef struct pollfd rd_pollfd_t;
+
+/** @brief poll(2) */
+#define rd_socket_poll(POLLFD,FDCNT,TIMEOUT_MS) poll(POLLFD,FDCNT,TIMEOUT_MS)
+
 /**
  * @brief Set socket to non-blocking
  * @returns 0 on success or errno on failure.
@@ -174,7 +212,7 @@ static RD_UNUSED int rd_fd_set_nonblocking (int fd) {
  * @brief Create non-blocking pipe
  * @returns 0 on success or errno on failure
  */
-static RD_UNUSED int rd_pipe_nonblocking (int *fds) {
+static RD_UNUSED int rd_pipe_nonblocking (rd_socket_t *fds) {
         if (pipe(fds) == -1 ||
             rd_fd_set_nonblocking(fds[0]) == -1 ||
             rd_fd_set_nonblocking(fds[1]))
