@@ -24,8 +24,8 @@ done
 
 TEST=./test-runner
 
-if [ ! -z "$2" ]; then
-    MODES=$2
+if [ ! -z "$1" ]; then
+    MODES=$1
 else
     MODES="bare"
     # Enable valgrind:
@@ -77,6 +77,11 @@ for mode in $MODES; do
 		$TEST	$ARGS
 	    RET=$?
 	    ;;
+        callgrind)
+	    valgrind $VALGRIND_ARGS --tool=callgrind $SUPP $GEN_SUPP \
+		$TEST	$ARGS
+	    RET=$?
+	    ;;
         gdb)
             if [[ -f gdb.run ]]; then
                 gdb -x gdb.run $ARGS $TEST
